// GSAP CONFIGS
gsap.registerPlugin( CustomEase );
CustomEase.create( "myEaseSmooth", "M0,0 C0.34,0 0.11,1 1,1" );
CustomEase.create( "myEaseSwift", "M0,0 C0.89,0 0.34,1 1,1" );
CustomEase.create( "cubic", "0.65,0.05,0.36,1" );

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// BEGIN – GLSL SHADERS ////////////////////////////////////////////////////////////////////////////////////////////

// Initializing GLSL
const glsl = ( x ) => x[ 0 ];

//Vertex
const vertex = glsl`
  // Varyings
  varying vec2 vUv;

  // Uniforms
  uniform vec2 uMouse;
  uniform vec4 uResolution;

  uniform float uTime;
  uniform float uDirection;
  uniform float uProgress;

  void main() {

      vUv = uv;
      gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.);
  }
`;

// Fragment
const fragment = glsl`
  // Varyings
  varying vec2 vUv;

  // Uniforms
  uniform vec2 uMouse;
  uniform vec4 uResolution;

  uniform float uTime;
  uniform float uProgress;
  uniform float uDirection;
  uniform float uMouseSpeed;

  // Uniforms Images
  uniform sampler2D uTexture;
  uniform sampler2D uMap;

  void main() {

    vec4 color = texture2D(uTexture, vUv);
    gl_FragColor = vec4(color);
  }
`;

// END – GLSL SHADERS ///////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MAIN SETUP /////////////////////////////////////////////////////////////////////////////////////////////////

const container = document.body;
const renderer = new THREE.WebGLRenderer( { alpha: true } );

renderer.setSize( window.innerWidth, window.innerHeight );
renderer.setPixelRatio( Math.min( window.devicePixelRatio, 2 ) );

container.append( renderer.domElement );

const scene = new THREE.Scene();

const camera = new THREE.PerspectiveCamera(
	45,
	window.innerWidth / window.innerHeight,
	0.1,
	100
);

camera.position.z = 5;

let time = 0;

// Store mouse position
const mouse = new THREE.Vector2();
const previousMouse = new THREE.Vector2();
let mouseSpeed = 0;
let smoothMouseSpeed = 0;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// IMPORTANT FUNCTIONS /////////////////////////////////////////////////////////////////////////////////////////////

// Get camera FOV
function getCameraFOV() {

	const height = window.innerHeight;
	const fov = Math.atan( height / ( 2 * camera.position.z ) ) * 2;
	return ( fov / Math.PI ) * 180;

}

// Set mesh scale
function setMeshScale( mesh ) {

	const texture = mesh.material.uniforms.uTexture.value;
	const image = texture.image;
	const imgRect = image.getBoundingClientRect();

	mesh.scale.set( imgRect.width, imgRect.height, 1 );
	mesh.userData.imgRect = imgRect;

}

// Set mesh position
function setMeshPosition( mesh, y = window.scrollY ) {

	const texture = mesh.material.uniforms.uTexture.value;
	const image = texture.image;
	const imgRect = image.getBoundingClientRect();

	mesh.position.set(
		image.offsetLeft - window.innerWidth / 2 + imgRect.width / 2,
		- image.offsetTop + window.innerHeight / 2 - imgRect.height / 2 + y,
		0
	);

}

// Set mesh's scale and position
function setScalePosition( mesh ) {

	setMeshScale( mesh );
	setMeshPosition( mesh );

}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SCENE OBJECTS ///////////////////////////////////////////////////////////////////////////////////////////////////

const loader = new THREE.TextureLoader();
const requests = [
	loader.loadAsync( 'images/bl-image-2.jpg' ),
	loader.loadAsync( 'images/bl-image-3.jpg' ),
	loader.loadAsync( 'images/wk-image-1.jpg' ),
	loader.loadAsync( 'images/wk-image-2.jpg' ),
	loader.loadAsync( 'images/wk-image-3.jpg' ),
	loader.loadAsync( 'images/wk-image-4.jpg' )
];

const allMeshes = [];
const geometry = new THREE.PlaneBufferGeometry();

function htmlToWebgl() {

	Promise.all( requests ).then( ( textures ) => {

		textures.forEach( ( texture ) => {

			const material = new THREE.ShaderMaterial( {
				uniforms: {
					uTime: { type: "f", value: 0 },
					uDirection: { type: "f", value: 0 },
					uProgress: { type: "f", value: 0 },
					uMouseSpeed: { type: "f", value: 0 },
					uMouse: { type: "v2", value: new THREE.Vector2( 0, 0 ) },
					uTexture: { value: texture },
					uResolution: { type: "v4", value: new THREE.Vector4() },
				},

				side: THREE.DoubleSide,
				vertexShader: vertex,
				fragmentShader: fragment,
			} );

			const mesh = new THREE.Mesh( geometry, material );

			//setScalePosition( mesh );
			allMeshes.push( mesh );

			scene.add( mesh );

		} );

	} );



}

htmlToWebgl();

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// HELLO CUBE //////////////////////////////////////////////////////////////////////////////////////////////////////

// Hello Cube
let cubeMesh;

function helloCube() {

	const geometry = new THREE.BoxBufferGeometry();
	const material = new THREE.MeshNormalMaterial();
	cubeMesh = new THREE.Mesh( geometry, material );
	cubeMesh.scale.set( 10, 10, 10 );
	scene.add( cubeMesh );

}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MOUSE POSITIONS /////////////////////////////////////////////////////////////////////////////////////////////////

// Get mouse position
function mouseMoveEvent() {

	document.addEventListener( "mousemove", ( position ) => {

		mouse.x = position.x / container.offsetWidth;
		mouse.y = 1 - position.y / container.offsetHeight;

		// console.log(position);

		allMeshes.forEach( ( mesh ) => {

			mesh.material.uniforms.uMouse.value = mouse;

		} );

	} );

}

// Get mouse Speed
function mouseVelocity() {

	mouseSpeed = Math.sqrt(
		( previousMouse.x - mouse.x ) ** 2 + ( previousMouse.y - mouse.y ) ** 2
	);

	smoothMouseSpeed += 0.1 * ( mouseSpeed - smoothMouseSpeed );

	previousMouse.x = mouse.x;
	previousMouse.y = mouse.y;

}

mouseMoveEvent();

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// RESIZE SCENE ////////////////////////////////////////////////////////////////////////////////////////////////////

// Resize
function resize() {

	renderer.setSize( window.innerWidth, window.innerHeight );
	camera.aspect = window.innerWidth / window.innerHeight;
	camera.updateProjectionMatrix();

	// Resize Mesh
	allMeshes.forEach( ( mesh ) => setScalePosition( mesh ) );

	// Image aspect ratio
	const imageAspect = 853 / 1280;
	let a1, a2;
	if ( window.innerWidth / window.innerHeight > imageAspect ) {

		a1 = ( window.innerWidth / window.innerHeight ) * imageAspect;
		a2 = 1;

	} else {

		a1 = 1;
		a2 = ( window.innerWidth / window.innerHeight ) * imageAspect;

	}

	allMeshes.forEach( ( mesh ) => {

		mesh.material.uniforms.uResolution.value.x = container.offsetWidth;
		mesh.material.uniforms.uResolution.value.y = container.offsetHeight;
		mesh.material.uniforms.uResolution.value.z = a1;
		mesh.material.uniforms.uResolution.value.w = a2;

	} );

	// // Mesh aspect ration
	// if (container.offsetWidth / container.offsetHeight > 1) {
	//   // Use this for landscape images
	//   allMeshes.forEach((mesh) => {
	//     mesh.scale.x = camera.aspect;
	//   });

	//   // // Use this for portrait images
	//   // allMeshes.forEach((mesh) => {
	//   //   mesh.scale.y = camera.aspect;
	//   // });
	// } else {
	//   allMeshes.forEach((mesh) => {
	//     mesh.scale.y = 1 / camera.aspect;
	//   });
	// }

}

resize();

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EVENTS //////////////////////////////////////////////////////////////////////////////////////////////////////////

function meshEntry() {

	allMeshes.forEach( ( mesh ) => {

		gsap.from( mesh.material.uniforms.uTexture, {
			opacity: 0,
			//y: 144,
			duration: 1.34,
			ease: "myEaseSmooth",
			stagger: 0.08,
		} );

	} );

}

// Window resize
window.addEventListener( "resize", resize );

// Scroll Event
window.addEventListener( "scroll", ( e ) => {

	allMeshes.forEach( ( mesh ) => setMeshPosition( mesh, window.scrollY ) );

} );

window.addEventListener( "load", ( e ) => {

	meshEntry();

} );

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// RENDER SCENE ////////////////////////////////////////////////////////////////////////////////////////////////////

// Render scene
function render() {

	// // Hello cuber motion
	// cubeMesh.rotation.x += 0.01;
	// cubeMesh.rotation.y += 0.01;

	// All other elements
	time += 0.05;
	mouseVelocity();

	allMeshes.forEach( ( mesh ) => {

		mesh.material.uniforms.uMouseSpeed.value = smoothMouseSpeed;
		mesh.material.uniforms.uTime.value = time;

	} );

	// Request & Render
	requestAnimationFrame( render );
	renderer.render( scene, camera );

}

// helloCube();
render();

// END OF THREE.JS SETUP ///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
